@@ -5,7 +5,7 @@
     Copyright (C) 2007 Giuseppe "denever" Martino
     begin                : Fri 23 Mar 2007
     email                : denever@users.sourceforge.net
- ***************************************************************************/
+***************************************************************************/
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -86,7 +86,7 @@ CheckBox::CheckBox():
     m_cols_count(0),
     m_strings_lasted(0),
     m_frm_check(cmpmsg_title),
-    m_frm_legend("Legend"),
+    m_frm_input("Insert copied signs"),    
     m_frm_overall(ovrmsg_title),
     m_frm_smbrate(smbmsg_title),
     m_lbl_legend(chkmsg_legend)
@@ -111,11 +111,12 @@ CheckBox::CheckBox():
     m_ali_overall.add(m_prb_overall);
     m_frm_overall.add(m_ali_overall);    
     pack_start(m_frm_overall, Gtk::PACK_SHRINK);
-    
-    m_frm_legend.add(m_lbl_legend);
-    pack_start(m_frm_legend, Gtk::PACK_SHRINK);
 
-    pack_start(m_txt_copied, Gtk::PACK_SHRINK);
+    m_box_input.pack_start(m_lbl_legend);
+    m_box_input.pack_start(m_txt_copied);
+    m_frm_input.add(m_box_input);
+    
+    pack_start(m_frm_input, Gtk::PACK_SHRINK);
 
     m_txt_copied.signal_return_pressed().connect( sigc::mem_fun(*this, &CheckBox::on_txt_copied_return_pressed) );
 }
@@ -125,15 +126,18 @@ CheckBox::~CheckBox()
 
 void CheckBox::append_copied(Glib::ustring copied)
 {
-  Gtk::TreeModel::Row row = *(m_ref_string->append());
-  row[m_mod_string.m_col_copied] = padding(m_group_size, copied);
+    Gtk::TreeModel::Row row = *(m_ref_string->append());
+    row[m_mod_string.m_col_copied] = padding(m_group_size, copied);
 }
 
-void CheckBox::append_symbol(char symbol, unsigned int percentage)
+void CheckBox::append_symbol(char symbol, unsigned int keyed, unsigned int copied, unsigned int missed, unsigned int percentage)
 {
-  Gtk::TreeModel::Row row = *(m_ref_symbol->append());
-  row[m_mod_symbol.m_col_symbol] = Glib::ustring(1, symbol);
-  row[m_mod_symbol.m_col_percentage] = percentage;
+    Gtk::TreeModel::Row row = *(m_ref_symbol->append());
+    row[m_mod_symbol.m_col_symbol] = Glib::ustring(1, symbol);
+    row[m_mod_symbol.m_col_keyed] = keyed;
+    row[m_mod_symbol.m_col_copied] = copied;
+    row[m_mod_symbol.m_col_missed] = missed;
+    row[m_mod_symbol.m_col_percentage] = percentage;
 }
 
 
@@ -187,7 +191,7 @@ void CheckBox::on_exercise_finished(std::list<std::string> lst)
     }
 
     double fraction = 1 - double(wrong_letters)/double(m_total_chars);    
-    double overall_percentage = 100*fraction;
+    double overall_percentage = int(100*fraction);
 
     m_prb_overall.set_fraction(fraction);
     Glib::ustring text = Glib::Ascii::dtostr(overall_percentage) + "%";
@@ -245,7 +249,7 @@ void CheckBox::on_exercise_finished(std::list<std::string> lst)
 	char c = (*mtc).first;
 	
 	if (keyed_all[c] != 0)
-	    append_symbol(c,int(100*copied_good[c]/keyed_all[c]));
+	    append_symbol(c,keyed_all[c], copied_good[c], keyed_missed[c], int(100*copied_good[c]/keyed_all[c]));
     }
 }
 
@@ -284,6 +288,9 @@ void CheckBox::prepare_scl_symbol()
     m_trv_symbol.set_model(m_ref_symbol);
 
     m_cols_count = m_trv_symbol.append_column(smbmsg_symbol, m_mod_symbol.m_col_symbol);
+    m_cols_count = m_trv_symbol.append_column("Keyed", m_mod_symbol.m_col_keyed);
+    m_cols_count = m_trv_symbol.append_column("Copied", m_mod_symbol.m_col_copied);
+    m_cols_count = m_trv_symbol.append_column("Missed", m_mod_symbol.m_col_missed);            
 
     symbol_cell = new Gtk::CellRendererProgress;
 
@@ -300,33 +307,3 @@ void CheckBox::prepare_scl_symbol()
 #endif
     }
 }
-
-/*
-void display_symbol_rate(const list<string>& lks, const list<string>& lcs)
-{
-    map<char, unsigned int> keyed_all;
-    map<char, unsigned int> keyed_bad;
-    map<char, unsigned int> keyed_missed;
-    map<char, unsigned int> copied_all;
-    map<char, unsigned int> copied_good;
-    map<char, unsigned int> copied_bad;
-
-    // compute results
-
-    // iterate over the lists of strings
-    c_lststr lks_it;
-    c_lststr lcs_it;
-
-    for(lks_it = lks.begin(), lcs_it = lcs.begin();
-        lks_it != lks.end() && lcs_it != lcs.end();
-        ++lks_it, ++lcs_it
-       )
-    {
-	// ensure that the copied string is as long as the keyed string,
-	// padding it with 'MISSED_MARKER' if necessary
-	string copied = padding((*lks_it).size(), *lcs_it);
-
-	// iterate over the characters in each group
-    }
-}
-*/
