@@ -31,6 +31,8 @@
 
 using namespace gtkmmorsegui;
 
+const char* patterns = "hf";
+
 PatternExcBox::PatternExcBox(Glib::RefPtr<Gnome::Conf::Client> conf_client):
     m_conf_client(conf_client),
     m_frm_msg("Exercise explanation"),
@@ -76,7 +78,6 @@ PatternExcBox::PatternExcBox(Glib::RefPtr<Gnome::Conf::Client> conf_client):
 	    for(unsigned int j = 0; j < 5; j++)
 		m_rbt_check[a][i][j].set_tooltips("First pattern", "Second pattern");
 
-
     for(unsigned int i = 0; i < 4; i++)
 	for(unsigned int j = 0; j < 5; j++)
 	    m_tbl_check1.attach(m_rbt_check[0][i][j], j, j+1, i, i+1, Gtk::SHRINK, Gtk::SHRINK);
@@ -99,12 +100,15 @@ PatternExcBox::PatternExcBox(Glib::RefPtr<Gnome::Conf::Client> conf_client):
 
     m_box_check.pack_start(m_tbl_lbls);    
     m_box_check.pack_start(m_tbl_check1);
+    m_box_check.pack_start(m_vsp_1);
     m_box_check.pack_start(m_tbl_check2);
+    m_box_check.pack_start(m_vsp_2);    
     m_box_check.pack_start(m_tbl_check3);
 
     m_frm_checkboard.add(m_box_check);
 
     pack_start(m_frm_checkboard, Gtk::PACK_SHRINK);
+    pack_start(m_prb_overall, Gtk::PACK_SHRINK);
 }
 
 PatternExcBox::~PatternExcBox()
@@ -141,7 +145,7 @@ void PatternExcBox::on_btn_firstpattern_clicked()
     
     m_audioout->enqueue_pause(1000);
     
-    current_keyer << (unsigned char)'f';
+    current_keyer << (unsigned char)patterns[0];
     
     m_audioout->play();
 
@@ -169,7 +173,7 @@ void PatternExcBox::on_btn_seconpattern_clicked()
     
     m_audioout->enqueue_pause(1000);
     
-    current_keyer << (unsigned char)'h';
+    current_keyer << (unsigned char)patterns[1];
     
     m_audioout->play();
 
@@ -178,9 +182,17 @@ void PatternExcBox::on_btn_seconpattern_clicked()
 
 void PatternExcBox::on_btn_start_clicked()
 {
+    for(unsigned int a = 0; a < 3; a++)
+	for(unsigned int i = 0; i < 4; i++)
+	    for(unsigned int j = 0; j < 5; j++)
+		m_rbt_check[a][i][j].clear();
+
     m_btn_start.set_sensitive(false);
     m_frm_checkboard.set_sensitive(true);
 
+    m_prb_overall.set_fraction(0);
+    m_prb_overall.set_text("");	    
+		    
     unsigned int begin_pause = (unsigned int) m_conf_client->get_float("/apps/gtkmmorse/keyer/beginpause");
     unsigned int keyspeed = 12;
     unsigned int charpause = 3;
@@ -195,7 +207,7 @@ void PatternExcBox::on_btn_start_clicked()
     
     libkeyer::Keyer current_keyer(m_audioout, keyspeed, charpause, strpause, 900, 850);
 
-    libexercises::Blocks pattern_exc(strnum, "hf", 5);
+    libexercises::Blocks pattern_exc(strnum, patterns, 5);
 
     m_exercise_strings = pattern_exc.stringtok();
     
@@ -216,6 +228,43 @@ void PatternExcBox::on_play_finished()
     m_btn_seconpattern.set_sensitive(true);    
     m_frm_checkboard.set_sensitive(false);
     m_finished.emit(m_exercise_strings);    
+    
+  // Check exercise
+    typedef std::list< std::string >::const_iterator c_vec;
+    typedef std::string::const_iterator c_str;
+
+    c_vec vit = m_exercise_strings.begin(); 
+
+    unsigned int copied_good = 0;
+    
+    for(unsigned int a = 0; a < 3; a++)
+	for(unsigned int i = 0; i < 4; i++)
+	{
+	    c_str sit = (*vit).begin();
+	    
+	    for(unsigned int j = 0; j < 5; j++)
+	    {
+		char c = *sit;		
+		unsigned int pattern = m_rbt_check[a][i][j].choice();
+		if( c == patterns[pattern])
+		{
+		    m_rbt_check[a][i][j].set_ok();
+		    copied_good++;
+		}
+		else
+		    m_rbt_check[a][i][j].set_no();
+		    
+		sit++;
+	    }
+	    vit++;
+	}
+
+    double fraction = double(copied_good)/double(60);    
+    double overall_percentage = int(100*fraction);
+
+    m_prb_overall.set_fraction(fraction);
+    Glib::ustring text = Glib::Ascii::dtostr(overall_percentage) + "%";
+    m_prb_overall.set_text(text);
 }
 
 void PatternExcBox::on_firstpattern_finished()
